@@ -1,4 +1,4 @@
-/* Copyright (c) 2000 Thorsten Kukuk
+/* Copyright (c) 2000, 2002 Thorsten Kukuk
    This file is part of ypbind-mt.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -20,6 +20,8 @@
 #include "config.h"
 #endif
 
+#define _GNU_SOURCE
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <syslog.h>
@@ -47,11 +49,11 @@ log_msg (int type, const char *fmt,...)
     {
 #ifndef HAVE_VSYSLOG
       vsnprintf (msg, 512, fmt, ap);
-      syslog (LOG_NOTICE, "%s", msg);
+      syslog (type, "%s", msg);
 #else
-      vsyslog (LOG_NOTICE, fmt, ap);
+      vsyslog (type, fmt, ap);
 #endif
     }
-  
+
   va_end (ap);
 }
